@@ -225,15 +225,17 @@ public:
 							if (res || err == ERROR_IO_PENDING) {
 								continue;
 							}
+							in_.reset();
 							write_buffer_.clear();
 							write_error_ = rwresult{ rwresult::other, err };
 						}
 					}
 					else {
 						DWORD err = GetLastError();
-						if (err == ERROR_IO_PENDING) {
+						if (err == ERROR_IO_PENDING || err == ERROR_IO_INCOMPLETE) {
 							continue;
 						}
+						in_.reset();
 						write_buffer_.clear();
 						write_error_ = rwresult{ rwresult::other, err };
 					}
@@ -408,16 +410,16 @@ public:
 			quit_ = false;
 
 			if (out_.read_ != INVALID_HANDLE_VALUE) {
-				CancelIo(out_.read_);
+				CancelIoEx(out_.read_, &ol_read_);
 				DWORD read{};
-				while (!GetOverlappedResult(out_.read_, &ol_read_, &read, false) && GetLastError() == ERROR_IO_PENDING) {
+				while (!GetOverlappedResult(out_.read_, &ol_read_, &read, false) && (GetLastError() == ERROR_IO_PENDING || GetLastError() == ERROR_IO_INCOMPLETE)) {
 					yield();
 				}
 			}
 			if (in_.write_ != INVALID_HANDLE_VALUE) {
-				CancelIo(in_.write_);
+				CancelIoEx(in_.write_, &ol_write_);
 				DWORD written{};
-				while (!GetOverlappedResult(in_.write_, &ol_write_, &written, false) && GetLastError() == ERROR_IO_PENDING) {
+				while (!GetOverlappedResult(in_.write_, &ol_write_, &written, false) && (GetLastError() == ERROR_IO_PENDING || GetLastError() == ERROR_IO_INCOMPLETE)) {
 					yield();
 				}
 			}
@@ -495,8 +497,6 @@ public:
 							if (err != ERROR_IO_PENDING) {
 								return rwresult{ rwresult::other, err };
 							}
-							waiting_read_ = true;
-							SetEvent(sync_);
 						}
 					}
 					return rwresult(len);
@@ -512,7 +512,7 @@ public:
 				}
 				else {
 					DWORD err = GetLastError();
-					if (err == ERROR_IO_PENDING) {
+					if (err == ERROR_IO_PENDING || err == ERROR_IO_INCOMPLETE) {
 						waiting_read_ = true;
 						SetEvent(sync_);
 						return rwresult{ rwresult::wouldblock, 0 };
@@ -559,7 +559,9 @@ public:
 				SetEvent(sync_);
 				return rwresult(len);
 			}
-			return rwresult{rwresult::other, err};
+			in_.reset();
+			write_error_ = rwresult{ rwresult::other, err };
+			return write_error_;
 		}
 		else {
 			DWORD written = 0;
